"""Test inner/outer classes

>>> outer.a
5
>>> outer.b
Traceback (most recent call last):
 ...
AttributeError: class outer has no attribute 'b'
>>> o = outer()
>>> o.a, o.b
(5, 6)
>>> o.foo()
5 5 6

#>>> outer.inner.a
Traceback (most recent call last):
 ...
AttributeError: class inner has no attribute 'a'
#>>> outer.inner.c
7
#>>> outer.inner.d
Traceback (most recent call last):
 ...
AttributeError: class inner has no attribute 'd'

#>>> i = outer.inner()
#>>> i.a
Traceback (most recent call last):
 ...
AttributeError: inner instance has no attribute 'a'
#>>> i.b
Traceback (most recent call last):
 ...
AttributeError: inner instance has no attribute 'b'

#>>> i.c
7
#>>> i.d
8
#>>> i.foo()
5 7 8
"""

class outer:
    a = 5

    def __init__(self):
        self.b = 6

    def foo(self):
        print outer.a, self.a, self.b

    class inner:
        """test inner classes.

        inv: self.c == 7
        """
        c = 7
        def __init__(self):
            self.d = 8

        def foo(self):
            print outer.a, self.c, self.d

def foo(a):
    """Test closures.

    pre: a != 5
    """
    def bar(b):
        """Test closures and inner functions.

        pre: b != 6
        """
        return a + b
    return bar(a)

def _test():
    import contract, doctest, testdbc2
    contract.checkmod(testdbc2)
    return doctest.testmod(testdbc2)

if __name__ == '__main__':
    t = _test()
    if t[0] == 0:
       print "test: %d tests succeeded" % t[1]
    else:
       print "test: %d/%d tests failed" % t
